@@ -18,17 +18,19 @@ package command
 
 import (
 	"github.com/containerd/containerd/defaults"
-	"github.com/containerd/containerd/services/server"
+	srvconfig "github.com/containerd/containerd/services/server/config"
 )
 
-func defaultConfig() *server.Config {
-	return &server.Config{
+func defaultConfig() *srvconfig.Config {
+	return &srvconfig.Config{
 		Root:  defaults.DefaultRootDir,
 		State: defaults.DefaultStateDir,
-		GRPC: server.GRPCConfig{
+		GRPC: srvconfig.GRPCConfig{
 			Address:        defaults.DefaultAddress,
 			MaxRecvMsgSize: defaults.DefaultMaxRecvMsgSize,
 			MaxSendMsgSize: defaults.DefaultMaxSendMsgSize,
 		},
+		DisabledPlugins: []string{},
+		RequiredPlugins: []string{},
 	}
 }
